<?php

namespace WPDesk\WooCommercePaymentStatus\Settings;

use WPDesk\WooCommercePaymentStatus\Conditional\Clusters\ConditionsCluster;
use WPDesk\WooCommercePaymentStatus\Conditional\ConditionalLogic\ConditionalLogicContainer;
use WPDesk\WooCommercePaymentStatus\Conditional\Conditions;
use WPDesk\WooCommercePaymentStatus\Conditional\ConditionsCreator;
use WPDesk\WooCommercePaymentStatus\Conditional\Groups\Group;
use WPDesk\WooCommercePaymentStatus\Settings\Compatibility\BackwardCompatibility;
use WPStatusVendor\WPDesk\Persistence\Adapter\WordPress\WordpressOptionsContainer;
use WPStatusVendor\WPDesk\Persistence\PersistentContainer;

class Settings {
	public const  SETTINGS_PREFIX               = 'order_statuses_';
	private const CONDITIONAL_LOGIC_OPTIONS_KEY = 'conditional_logic';

	public const  GENERAL_CONDITION_KEY         = 'wps_conditional_general_condition';
	public const  GENERAL_CONDITION_NEVER       = 'never';
	public const  GENERAL_CONDITION_ALWAYS      = 'always';
	public const  GENERAL_CONDITION_CONDITIONAL = 'conditional';
	private const OPTIONS_ORDER_STATUS_KEY      = 'order_status';
	private const OPTIONS_CLUSTERS_KEY          = 'clusters';
	private const OPTIONS_CONDITIONS_KEY        = 'conditions';
	private const OPTIONS_CONDITION_KEY         = 'condition';
	private const OPTIONS_VALUE_KEY             = 'value';
	private const OPTIONS_STATEMENT_KEY         = 'statement';

	private WordpressOptionsContainer $persistance;
	private ConditionsCreator $conditions_creator;

	public function __construct( WordpressOptionsContainer $persistance, ConditionsCreator $conditions_creator ) {
		$this->persistance        = $persistance;
		$this->conditions_creator = $conditions_creator;
	}

	public function get_settings(): PersistentContainer {
		return $this->persistance;
	}

	public function set( string $id, $value ): void {
		$this->persistance->set( $id, $value );
	}

	public function save_conditonal_logic( array $settings ): void {
		$this->set( self::CONDITIONAL_LOGIC_OPTIONS_KEY, $settings );
	}

	public function get_general_condition(): string {
		$general_condition = $this->get( self::GENERAL_CONDITION_KEY );

		if ( ! $general_condition ) {
			$backward_compatibility = new BackwardCompatibility( $this->conditions_creator );
			$general_condition      = $backward_compatibility->get_previous_settings_with_new_format();
		}

		return $general_condition ?? self::GENERAL_CONDITION_NEVER;
	}

	public function get_conditional_logic(): ConditionalLogicContainer {
		$conditional = new ConditionalLogicContainer();
		$data        = $this->get( self::CONDITIONAL_LOGIC_OPTIONS_KEY );

		if ( ! $data ) {
			$backward_compatibility = new BackwardCompatibility( $this->conditions_creator );

			if ( $backward_compatibility->is_previous_settings_with_conditions() ) {
				$backward_compatibility_group = $backward_compatibility->create_group();
				$conditional->add_group( $backward_compatibility_group );
			}

			return $conditional;
		}

		foreach ( $data as $group_id => $group_items ) {
			$group = new Group( $group_id );

			$group->set_target_status( $group_items[ self::OPTIONS_ORDER_STATUS_KEY ] );

			if ( empty( $group_items[ self::OPTIONS_CLUSTERS_KEY ] ) ) {
				continue;
			}
			foreach ( $group_items[ self::OPTIONS_CLUSTERS_KEY ] as $cluster_id => $cluster_items ) {
				$cluster = new ConditionsCluster( $cluster_id, $group_id );

				if ( empty( $cluster_items[ self::OPTIONS_CONDITIONS_KEY ] ) ) {
					continue;
				}

				foreach ( $cluster_items[ self::OPTIONS_CONDITIONS_KEY ] as $condition_id => $condition_items ) {

					$condition_slug = $condition_items[ self::OPTIONS_CONDITION_KEY ] ?? Conditions::GATEWAY_KEY;
					$condition      = $this->conditions_creator->create_condition( $group_id, $cluster_id, $condition_id, $condition_slug );

					$condition
						->set_value( $condition_items[ self::OPTIONS_VALUE_KEY ] ?? [] )
						->set_statement( $condition_items[ self::OPTIONS_STATEMENT_KEY ] );

					$cluster->add_condition( $condition );
				}

				$group->add_cluster( $cluster );
			}

			$conditional->add_group( $group );
		}

		return $conditional;
	}

	public function get( string $name ) {

		$default = $this->default_values[ $name ] ?? null;

		return $this->persistance->get_fallback( $name, $default );
	}
}
