<?php

namespace WPDesk\WooCommercePaymentStatus\UI;

use WPDesk\WooCommercePaymentStatus\Conditional\Conditions;
use WPDesk\WooCommercePaymentStatus\Conditional\ConditionsCreator;
use WPDesk\WooCommercePaymentStatus\UI\Elements\ClusterUIElement;
use WPDesk\WooCommercePaymentStatus\UI\Elements\ConditionUIElement;
use WPDesk\WooCommercePaymentStatus\UI\Elements\GroupUIElement;
use WPDesk\WooCommercePaymentStatus\UI\Fields\Select2Field;
use WPDesk\WooCommercePaymentStatus\UI\Fields\Select2SearchField;
use WPDesk\WooCommercePaymentStatus\UI\Fields\SelectField;
use WPStatusVendor\WPDesk\View\Renderer\Renderer;

class UIRenderer {

	protected Renderer $renderer;

	protected Conditions $available_conditions;
	protected ConditionsCreator $conditions_creator;

	public function __construct( Renderer $renderer, Conditions $available_conditions, ConditionsCreator $conditions_creator ) {
		$this->available_conditions = $available_conditions;
		$this->renderer             = $renderer;
		$this->conditions_creator   = $conditions_creator;
	}

	public function render( UIElement $element, $params = [] ): string {
		$template = $element->get_template_id();

		return $this->renderer->render( $template, $params );
	}

	public function render_condition( ConditionUIElement $condition_ui ): string {

		$condition_input_name = $this->create_condition_input_name( $condition_ui->get_group_id(), $condition_ui->get_cluster_id(), $condition_ui->get_condition_id() );

		$params                           = $condition_ui->get_params();
		$params['value_select_field']     = $this->render_available_values( $condition_ui->get_available_options(), $condition_ui->get_id(), $condition_ui->get_value(), $condition_input_name );
		$params['statement_select_field'] = $this->render_available_statements( $condition_ui->get_statements(), $condition_ui->get_statement(), $condition_input_name );

		return $this->render( $condition_ui, $params );
	}

	public function render_cluster( ClusterUIElement $cluster_ui ): string {

		$cluster_content = '';

		foreach ( $cluster_ui->get_conditions() as $condition ) {
			$condition_ui      = new ConditionUIElement( $condition, $this->available_conditions );
			$condition_content = $this->render_condition( $condition_ui );
			$cluster_content   .= $condition_content;
		}

		$params               = $cluster_ui->get_params();
		$params['conditions'] = $cluster_content;

		return $this->render( $cluster_ui, $params );
	}

	public function render_group( GroupUIElement $group_ui ): string {

		$group_content = '';

		foreach ( $group_ui->get_clusters() as $cluster ) {

			$cluster_ui = new ClusterUIElement( $cluster );

			$group_content .= $this->render_cluster( $cluster_ui );
		}

		$params             = $group_ui->get_params();
		$params['clusters'] = $group_content;

		return $this->render( $group_ui, $params );
	}

	protected function render_empty_condition( string $cluster_id, string $group_id ): string {

		$blank_condition = $this->conditions_creator->create_empty_condition( $group_id, $cluster_id );

		$condition_input_name = $this->create_condition_input_name( $blank_condition->get_group_id(), $blank_condition->get_cluster_id(), $blank_condition->get_condition_id() );

		$empty_logic = new ConditionUIElement( $blank_condition, $this->available_conditions );
		$params      = [
			'value_select_field'     => $this->render_available_values( $blank_condition->get_available_options(), $blank_condition->get_id(), $blank_condition->get_value(), $condition_input_name ),
			'statement_select_field' => $this->render_available_statements( $blank_condition->get_statements(), $blank_condition->get_statement(), $condition_input_name ),
		];

		return $this->render( $empty_logic, array_merge( $empty_logic->get_params(), $params ) );
	}

	protected function render_empty_cluster( string $group_id ): string {

		$blank_cluster = $this->conditions_creator->create_empty_cluster( $group_id );

		$cluster_ui = new ClusterUIElement( $blank_cluster );

		return $this->render_cluster( $cluster_ui );
	}

	public function render_empty_group(): string {

		$blank_group = $this->conditions_creator->create_empty_group();

		$group_ui_element = new GroupUIElement( $blank_group );

		return $this->render_group( $group_ui_element );
	}

	protected function render_available_statements( array $available_statements, string $selected_statement, string $condition_input_name ): string {

		$select_field = new SelectField();
		$select_field->set_available_options( $available_statements );
		$select_field->set_selected_option( [ $selected_statement ] );
		$select_field->set_class( 'wps-logic-condition__statement' );
		$select_field->set_name( $condition_input_name . '[statement]' );

		return $this->render( $select_field, $select_field->get_params() );
	}

	protected function render_available_values( array $available_options, string $condition_type, array $condition_value, string $condition_input_name ): string {

		$field = $this->get_select_field_object( $condition_type );

		$field->set_available_options( $available_options );
		$field->set_selected_option( $condition_value );
		$field->set_name( $condition_input_name . '[value][]' );

		return $this->render( $field, $field->get_params() );
	}

	protected function create_condition_input_name( string $group_id, string $cluster_id, string $condition_id ): string {
		return 'wps-conditional[' . $group_id . '][clusters][' . $cluster_id . '][conditions][' . $condition_id . ']';
	}

	protected function get_select_field_object( string $type ): SelectField {
		switch ( $type ) {
			case Conditions::PRODUCT_TYPE_KEY:
				$field = new SelectField();
				$field->set_class( 'wps-logic-condition__value' );
				break;
			case Conditions::PRODUCT_KEY:
				$field = new Select2SearchField();
				break;
			default:
				$field = new Select2Field();
				break;
		}

		return $field;
	}
}
